"""ModelAudit - LLM 蒸馏检测与模型指纹审计工具

白盒指纹（REEF CKA）+ 黑盒指纹（LLMmap）+ 风格分析，
统一接口审计模型来源与血缘关系。
"""

__version__ = "0.4.0"

from modelaudit.base import Fingerprinter
from modelaudit.models import (
    AuditResult,
    ComparisonResult,
    DetectionResult,
    Fingerprint,
)

__all__ = [
    "AuditResult",
    "ComparisonResult",
    "DetectionResult",
    "Fingerprint",
    "Fingerprinter",
    "__version__",
]
